/*
 * Watchdog.java
 *
 * Created on January 25, 2005, 6:33 PM
 */

package util;

import java.util.Vector;
//import java.util.LinkedList;


public class Watchdog {
    
    class WatchNode {
        Thread thread;
        WatchdogListener object;
        long wakeupTime;
        
        public WatchNode(Thread thread, long wakeupTime) {
            this.thread = thread;
            this.wakeupTime = wakeupTime;
        }
        
        public WatchNode(WatchdogListener object, long wakeupTime) {
            this.object = object;
            this.wakeupTime = wakeupTime;
        }
    }
    
    class WatchdogThread extends Thread {
        Watchdog watchdog;
        
        WatchdogThread(String name, Watchdog parent) {
            super(name);
            this.watchdog = parent;
        }
        
        public void run() {
            WatchNode n;
            long waitTime;
            while(!stop) {
                synchronized (watchdog) {
                    if (!watched.isEmpty()) {
                        n = (WatchNode) watched.firstElement();
                        if (System.currentTimeMillis() >= n.wakeupTime) {
                            watched.removeElementAt(0);
                            try {
                                if (n.object == null) {
                                    n.thread.interrupt();
                                } else {
                                    n.object.watchdogTimeout();
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    }
                    if (watched.isEmpty()) {
                        waitTime = -1;
                    } else {
                        waitTime = Math.max(0,
                                ((WatchNode)watched.firstElement()).wakeupTime
                                - System.currentTimeMillis());
                    }
                    
                    if (stop) {
                        break;
                    }
                    
                    try {
                        if (waitTime > 0) {
                            watchdog.wait(waitTime);
                        } else if (waitTime < 0) {
                            watchdog.wait();
                        }
                    } catch (InterruptedException e) {}
                }
            }
        }
    }
    
    WatchdogThread watchdogThread;
    Vector watched = new Vector();
    boolean stop;
    
    /** Creates a new instance of Watchdog */
    public Watchdog() {
        watchdogThread = new WatchdogThread("WatchdogThread", this);
//        watchdogThread.setDaemon(true);
        watchdogThread.setPriority(3);
        watchdogThread.start();
    }
    
    /**
     * Schedules the calling <code>Thread</code> to be interrupted after the
     * requested time. The thread may cancel this request by calling
     * <code>cancelInterrupt</code>.
     */
    public synchronized void interruptAfter(long millis) {
        Thread toInterrupt = Thread.currentThread();
        WatchNode node = null;
        long wakeupTime = System.currentTimeMillis() + millis;
        for (int i=0; i < watched.size(); i++) {
            WatchNode n = (WatchNode) watched.elementAt(i);
            if (n.thread == toInterrupt) {
                node = n;
                break;
            }
        }
        
        if (node != null) {
            watched.removeElement(node);
            node.wakeupTime = wakeupTime;
        } else {
            node = new WatchNode(toInterrupt, wakeupTime);
        }
        
        addNode(node);
    }
    
    /**
     * Schedules the calling <code>Thread</code> to be interrupted after the
     * requested time. The thread may cancel this request by calling
     * <code>cancelInterrupt</code>.
     */
    public synchronized void callAfter(WatchdogListener toInterrupt,
            long millis) {
        WatchNode node = null;
        long wakeupTime = System.currentTimeMillis() + millis;
        for (int i=0; i < watched.size(); i++) {
            WatchNode n = (WatchNode) watched.elementAt(i);
            if (n.object == toInterrupt) {
                node = n;
                break;
            }
        }
        
        if (node != null) {
            watched.removeElement(node);
            node.wakeupTime = wakeupTime;
        } else {
            node = new WatchNode(toInterrupt, wakeupTime);
        }
        
        addNode(node);
    }
    
    private synchronized void addNode(WatchNode node) { //synch not needed - from caller
        int i;
        for (i=0; i < watched.size(); i++) {
            WatchNode n = (WatchNode) watched.elementAt(i);
            if (node.wakeupTime < n.wakeupTime) {
                watched.insertElementAt(node, i);
                if (i == 0) {
                    synchronized (watchdogThread) {
                        notify();
                    }
                }
                break;
            }
        }
        
        //Handles cases of empty list, and having to add at end of list
        if (i == (watched.size())) {
            watched.addElement(node);
            if (watched.size() == 1) {
                synchronized (watchdogThread) {
                    notify();
                }
            }
        }
    }
    
    public synchronized void cancelInterrupt() {
        Thread toInterrupt = Thread.currentThread();
        WatchNode node = null;
        for (int i=0; i < watched.size(); i++) {
            WatchNode n = (WatchNode) watched.elementAt(i);
            if (n.thread == toInterrupt) {
                node = n;
                break;
            }
        }
        if (node != null) {
            watched.removeElement(node);
        }
    }
    
    public synchronized void cancelCallback(WatchdogListener toCancel) {
        WatchNode node = null;
        for (int i=0; i < watched.size(); i++) {
            WatchNode n = (WatchNode) watched.elementAt(i);
            if (n.object == toCancel) {
                node = n;
                break;
            }
        }
        if (node != null) {
            watched.removeElement(node);
        }
    }
    
    public void stop() {
        stop = true;
        watchdogThread.interrupt();
    }
    
    public String toString() {
        String s = "Watchdog watching " + watched.size() + " threads/objects.";
//        for (WatchNode n: watched) {
//            s += "\n\twakeupTime: " + n.wakeupTime;
//        }
//        s += "\n";
        return s;
    }
}
